@@ -4,7 +4,7 @@
 """
 Copyright (C) 2008-2016 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
-SPDX-License-Identifier: GPL-2.0
+SPDX-License-Identifier: GPL-2.0-only
 
 
 
@@ -42,10 +42,7 @@ from configparser import ConfigParser, NoSectionError, NoOptionError
 
 # pylint: disable=wildcard-import,unused-wildcard-import
 from qt import *
-from qtpy import QT5, QT6, PYSIDE2, PYSIDE6, QT_VERSION, API_NAME, PYQT_VERSION
-if QT6:
-    # pylint:disable=no-name-in-module
-    from qtpy.QtCore import QKeyCombination  # type: ignore
+from qtpy import QT6, PYSIDE6, QT_VERSION, API_NAME, PYQT_VERSION
 
 # pylint: disable=wrong-import-position
 
@@ -71,7 +68,7 @@ class KApplication(QApplication):
 
     def __init__(self) ->None:
         assert not Internal.isServer, 'KApplication is not supported for the server'
-        QApplication.__init__(self, sys.argv)
+        super().__init__(sys.argv)
 
         # Qt uses sys.argv[0] as application name
         # which is used by QStandardPaths - if we start kajongg.py directly,
@@ -103,20 +100,14 @@ class KApplication(QApplication):
         self.translators.append(_)
         self.installTranslator(_)
         for language in reversed(list(MLocale.extendRegionLanguages(MLocale.currentLanguages()))):
-            _ = QLibraryInfo.location(QLibraryInfo.TranslationsPath)  # type:ignore[attr-defined]
-            # qtpy maps location() to path() for Qt6
+            _ = QLibraryInfo.path(QLibraryInfo.LibraryPath.TranslationsPath)
             self.installTranslatorFile(os.path.join( _, f'qtbase_{language}.qm'))
             self.installTranslatorFile(f'/usr/share/locale/{language}/LC_MESSAGES/kwidgetsaddons5_qt.qm')
 
     @classmethod
     def desktopSize(cls) -> QSize:
         """The size of the current screen"""
-        try:
-            result = Internal.app.desktop().availableGeometry()
-        except AttributeError:
-            assert Internal.mainWindow
-            result = Internal.mainWindow.screen().availableGeometry()
-        return result
+        return QGuiApplication.primaryScreen().availableGeometry()
 
 
 class CaptionMixin:
@@ -169,12 +160,13 @@ class IconLabel(QLabel):
     """for use in messages and about dialog"""
 
     def __init__(self, iconName:str, dialog:QDialog) ->None:
-        QLabel.__init__(self)
+        super().__init__()
         icon = KIcon(iconName)
         option = QStyleOption()
         option.initFrom(dialog)
-        self.setPixmap(icon.pixmap(dialog.style().pixelMetric(
-            QStyle.PixelMetric.PM_MessageBoxIconSize, option, dialog)))
+        style = dialog.style()
+        assert style
+        self.setPixmap(icon.pixmap(style.pixelMetric(QStyle.PixelMetric.PM_MessageBoxIconSize, option, dialog)))
 
 
 class KMessageBox:
@@ -227,7 +219,9 @@ class KMessageBox:
             scrollArea.setWidgetResizable(True)
             scrollArea.setFocusPolicy(Qt.FocusPolicy.NoFocus)
             scrollPal = QPalette(scrollArea.palette())
-            scrollArea.viewport().setPalette(scrollPal)
+            viewport = scrollArea.viewport()
+            if viewport:
+                viewport.setPalette(scrollPal)
             hLayout.addWidget(scrollArea, 5)
         else:
             hLayout.addWidget(messageLabel, 5)
@@ -253,32 +247,41 @@ class KDialog(CaptionMixin, QDialog):
     Close = QDialogButtonBox.StandardButton.Close
 
     def __init__(self, parent:Optional[QWidget]=None) ->None:
-        QDialog.__init__(self, parent)
+        super().__init__(parent)
         self.buttonBox = QDialogButtonBox()
         self.buttonBox.accepted.connect(self.accept)
         self.buttonBox.rejected.connect(self.reject)
         self.__mainWidget:Optional[QWidget] = None
 
+    def __set_text(self, buttonMask: QDialogButtonBox.StandardButton,
+        which:QDialogButtonBox.StandardButton, text:str) ->None:
+        """set button text"""
+        if which & buttonMask:
+            button = self.buttonBox.button(which)
+            if button:
+                button.setText(text)
+
+    def __connect_button(self, buttonMask: QDialogButtonBox.StandardButton,
+        which:QDialogButtonBox.StandardButton, connector:Any) ->None:
+        """set connector"""
+        if which & buttonMask:
+            button = self.buttonBox.button(which)
+            if button:
+                button.clicked.connect(connector)
+
     def setButtons(self, buttonMask:QDialogButtonBox.StandardButton) ->None:
         """(re)create the buttonbox and put all wanted buttons into it"""
         if not buttonMask:
             self.buttonBox.clear()
             return
         self.buttonBox.setStandardButtons(buttonMask)
-        if KDialog.Ok & buttonMask:
-            self.buttonBox.button(KDialog.Ok).setText(i18n('&OK'))
-        if KDialog.Apply & buttonMask:
-            self.buttonBox.button(KDialog.Apply).setText(i18n('&Apply'))
-        if KDialog.Cancel & buttonMask:
-            self.buttonBox.button(KDialog.Cancel).setText(i18n('&Cancel'))
-        if KDialog.Help & buttonMask:
-            self.buttonBox.button(KDialog.Help).setText(i18n('&Help'))
-        if KDialog.RestoreDefaults & buttonMask:
-            self.buttonBox.button(
-                KDialog.RestoreDefaults).setText(i18n('&Defaults'))
-            self.buttonBox.button(KDialog.RestoreDefaults).clicked.connect(self.restoreDefaults)
-        if KDialog.Help & buttonMask:
-            self.buttonBox.button(KDialog.Help).clicked.connect(Help.start)
+        self.__set_text(buttonMask, KDialog.Ok, i18n('&OK'))
+        self.__set_text(buttonMask, KDialog.Apply, i18n('&Apply'))
+        self.__set_text(buttonMask, KDialog.Cancel, i18n('&Cancel'))
+        self.__set_text(buttonMask, KDialog.Help, i18n('&Help'))
+        self.__set_text(buttonMask, KDialog.RestoreDefaults, i18n('&Defaults'))
+        self.__connect_button(buttonMask, KDialog.RestoreDefaults, self.restoreDefaults)
+        self.__connect_button(buttonMask, KDialog.Help, Help.start)
 
     def restoreDefaults(self) ->None:
         """virtual"""
@@ -287,17 +290,23 @@ class KDialog(CaptionMixin, QDialog):
         """see KDialog.setMainWidget"""
         if self.layout() is None:
             QVBoxLayout(self)
-            self.layout().addWidget(self.buttonBox)
+            _ = self.layout()
+            assert _ is not None
+            _.addWidget(self.buttonBox)
+        layout = self.layout()
+        assert layout is not None
         if self.__mainWidget:
-            self.layout().removeWidget(self.__mainWidget)
-            self.layout().removeWidget(self.buttonBox)
+            layout.removeWidget(self.__mainWidget)
+            layout.removeWidget(self.buttonBox)
         self.__mainWidget = widget
-        self.layout().addWidget(widget)
-        self.layout().addWidget(self.buttonBox)
+        layout.addWidget(widget)
+        layout.addWidget(self.buttonBox)
 
     def button(self, buttonCode:QDialogButtonBox.StandardButton) ->QPushButton:
         """return the matching button"""
-        return self.buttonBox.button(buttonCode)
+        result = self.buttonBox.button(buttonCode)
+        assert result
+        return result
 
     def returns(self, button:Optional['QDialogButtonBox.StandardButton']=None) ->Any:
         """the user answered"""
@@ -317,7 +326,7 @@ class KUser:
         """stub"""
         if sys.platform == 'win32':
             return self.loginName()
-        return pwd.getpwnam(self.loginName()).pw_gecos.replace(',', '')
+        return pwd.getpwnam(self.loginName()).pw_gecos.replace(',', '')  # pylint:disable=possibly-used-before-assignment
 
     @staticmethod
     def loginName() ->str:
@@ -405,7 +414,7 @@ class KStatusBar(QStatusBar):
     """stub"""
 
     def __init__(self, *args:Any, **kwargs:Any) ->None:
-        QStatusBar.__init__(self, *args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.__items:List[MyStatusBarItem] = []
 
     def hasItem(self, idx:int) ->bool:
@@ -438,7 +447,7 @@ class KXmlGuiWindow(CaptionMixin, QMainWindow):
     """stub"""
 
     def __init__(self) ->None:
-        QMainWindow.__init__(self)
+        super().__init__()
         self.actionStatusBar: 'Action'
         self.actionToolBar: 'Action'
         self.actionFullscreen: 'Action'
@@ -460,7 +469,9 @@ class KXmlGuiWindow(CaptionMixin, QMainWindow):
                 (i18n('&Help'), ('help', 'language', 'aboutkajongg'))):
             mainMenu = QMenu(menu)
             self.menus[menu] = (mainMenu, menuItems)
-            self.menuBar().addMenu(mainMenu)
+            menu_bar = self.menuBar()
+            if menu_bar:
+                menu_bar.addMenu(mainMenu)
         self.setCaption('')
         self.actionHelp = Action(self, "help", "help-contents", Help.start)
         self.actionHelp.setText(i18nc('@action:inmenu', '&Help'))
@@ -479,7 +490,7 @@ class KXmlGuiWindow(CaptionMixin, QMainWindow):
         """for mypy"""
         return cast(KStatusBar, QMainWindow.statusBar(self))
 
-    def showEvent(self, event:QShowEvent) ->None:
+    def showEvent(self, event:Optional[QShowEvent]) ->None:
         """now that the MainWindow code has run, we know all actions"""
         self.refreshToolBar()
         assert Internal.Preferences
@@ -488,14 +499,16 @@ class KXmlGuiWindow(CaptionMixin, QMainWindow):
         self.actionToolBar.setChecked(self.toolBar().isVisible())
         self.actionFullscreen.setChecked(
             self.windowState() & Qt.WindowState.WindowFullScreen == Qt.WindowState.WindowFullScreen)
-        QMainWindow.showEvent(self, event)
+        if event:
+            super().showEvent(event)
 
-    def hideEvent(self, event:QHideEvent) ->None:
+    def hideEvent(self, event:Optional[QHideEvent]) ->None:
         """save status"""
         assert Internal.Preferences
         Internal.Preferences.toolBarVisible = self.toolBar(
         ).isVisible()
-        QMainWindow.hideEvent(self, event)
+        if event:
+            super().hideEvent(event)
 
     def toggleStatusBar(self, checked:bool) ->None:
         """show / hide status bar"""
@@ -548,12 +561,13 @@ class KXmlGuiWindow(CaptionMixin, QMainWindow):
         """default"""
         return True
 
-    def closeEvent(self, event:QEvent) ->None:
+    def closeEvent(self, event:Optional[QEvent]) ->None:
         """call queryClose/queryExit"""
-        if self.queryClose() and self.queryExit():
-            event.accept()
-        else:
-            event.ignore()
+        if event:
+            if self.queryClose() and self.queryExit():
+                event.accept()
+            else:
+                event.ignore()
 
 
 class KConfigGroup:
@@ -628,7 +642,7 @@ class KConfig(ConfigParser):
     without support for a default section."""
 
     def __init__(self, path:Optional[str]=None) ->None:
-        ConfigParser.__init__(self, delimiters=('=', ))
+        super().__init__(delimiters=('=', ))
         if path is None:
             path = QStandardPaths.writableLocation(QStandardPaths.StandardLocation.AppConfigLocation)
             path = os.path.join(path, 'kajonggrc')
@@ -671,7 +685,7 @@ class Action(QAction):
 
     def __init__(self, parent:'KXmlGuiWindow', name:Optional[str]=None,
         icon:Optional[str]=None, slot:Optional[Callable]=None,
-        shortcut:Union[QKeySequence, Qt.Key, None]=None, actionData:Optional[Union[str, Type[QWidget]]]=None) ->None:
+        shortcut:Optional[str]=None, actionData:Optional[Union[str, Type[QWidget]]]=None) ->None:
         super().__init__(parent)
         if icon:
             self.setIcon(KIcon(icon))
@@ -682,12 +696,7 @@ class Action(QAction):
                 name = ''
             parent.actionCollection().addAction(name, self)
         if shortcut:
-            if QT6:
-                if isinstance(shortcut, QKeyCombination):
-                    shortcut = shortcut.key()
-                    assert shortcut
-            shortcut = cast(Qt.Key, shortcut)
-            self.setShortcut(QKeySequence(shortcut | Qt.KeyboardModifier.ControlModifier))  # type:ignore[operator]
+            self.setShortcut(QKeySequence(shortcut))
             self.setShortcutContext(Qt.ShortcutContext.ApplicationShortcut)
         if actionData is not None:
             self.setData(actionData)
@@ -731,7 +740,7 @@ class ItemBool(KConfigSkeletonItem):
     """boolean preferences setting used by KOnfigSkeleton"""
 
     def __init__(self, skeleton:'KConfigSkeleton', key:str, value:bool, default:bool) ->None:
-        KConfigSkeletonItem.__init__(self, skeleton, key, value, default)
+        super().__init__(skeleton, key, value, default)
 
     def getFromConfig(self) ->None:
         """if not there, use default"""
@@ -748,7 +757,7 @@ class ItemString(KConfigSkeletonItem):
     def __init__(self, skeleton:'KConfigSkeleton', key:str, value:str, default:str) ->None:
         if value == '':
             value = default
-        KConfigSkeletonItem.__init__(self, skeleton, key, value, default)
+        super().__init__(skeleton, key, value, default)
 
 
 class ItemInt(KConfigSkeletonItem):
@@ -756,7 +765,7 @@ class ItemInt(KConfigSkeletonItem):
     """integer preferences setting used by KOnfigSkeleton"""
 
     def __init__(self, skeleton:'KConfigSkeleton', key:str, value:int, default:int) ->None:
-        KConfigSkeletonItem.__init__(self, skeleton, key, value, default)
+        super().__init__(skeleton, key, value, default)
         self.minValue = -99999
         self.maxValue = 99999999
 
@@ -782,7 +791,7 @@ class KConfigSkeleton(QObject):
     configChanged = Signal()
 
     def __init__(self) ->None:
-        QObject.__init__(self)
+        super().__init__()
         self.currentGroup:str
         self.items:List[KConfigSkeletonItem] = []
         self.toolBarActions:str
@@ -969,7 +978,7 @@ class KLanguageButton(QWidget):
         """self and children"""
         self.button.deleteLater()
         self.popup.deleteLater()
-        QWidget.deleteLater(self)
+        super().deleteLater()
 
     def setText(self, txt:str) ->None:
         """proxy: sets the button text"""
@@ -1022,7 +1031,7 @@ class AboutKajonggDialog(KDialog):
         # pylint: disable=too-many-locals, too-many-statements
         from twisted import __version__
 
-        KDialog.__init__(self, parent)
+        super().__init__(parent)
         self.setCaption(i18n('About Kajongg'))
         self.setButtons(KDialog.Close)
         vLayout = QVBoxLayout()
@@ -1039,9 +1048,6 @@ class AboutKajonggDialog(KDialog):
         underVersions = ['Qt' + QT_VERSION +' API=' + API_NAME]
         if PYQT_VERSION:
             underVersions.append('sip ' + SIP_VERSION_STR)
-        if PYSIDE2:
-            import PySide2
-            underVersions.append('PySide2 ' + PySide2.__version__)
         if PYSIDE6:
             import PySide6
             underVersions.append('PySide6 ' + PySide6.__version__)
@@ -1090,9 +1096,9 @@ class AboutKajonggDialog(KDialog):
                 "(C) 2008-2017 Wolfgang Rohdewald",
                 '<a href="https://apps.kde.org/kajongg">https://apps.kde.org/kajongg</a>']))
         licenseLabel = QLabel()
+        licenseLabel.setOpenExternalLinks(True)
         licenseLabel.setText(
-            f'<a href="file://{self.licenseFile()}">GNU General Public License Version 2</a>')
-        licenseLabel.linkActivated.connect(self.showLicense)
+            '<a href="https://spdx.org/licenses/GPL-2.0-only.html">GNU General Public License Version 2</a>')
         aboutLayout.addWidget(aboutLabel)
         aboutLayout.addWidget(licenseLabel)
         aboutWidget.setLayout(aboutLayout)
@@ -1127,65 +1133,6 @@ class AboutKajonggDialog(KDialog):
         self.setLayout(vLayout)
         self.buttonBox.setFocus()
 
-    @staticmethod
-    def licenseFile() ->Optional[str]:
-        """which may currently only be 1: GPL_V2"""
-        if QT5:
-            # pyqt 2.3.0 does not seem to handle this
-            prefix = QLibraryInfo.location(QLibraryInfo.LibraryLocation.PrefixPath)  # type:ignore[attr-defined]
-        else:
-            prefix = QLibraryInfo.path(QLibraryInfo.LibraryPath.PrefixPath)  # type:ignore[attr-defined]
-        for path in ('COPYING', '../COPYING',
-                     f'{prefix}/share/kf5/licenses/GPL_V2'):
-            path = os.path.abspath(path)
-            if os.path.exists(path):
-                return path
-        return None
-
-    @classmethod
-    def showLicense(cls) ->None:
-        """as the name says"""
-        assert Internal.mainWindow
-        LicenseDialog(Internal.mainWindow, cls.licenseFile()).exec()
-
-
-class LicenseDialog(KDialog):
-
-    """see kaboutapplicationdialog.cpp"""
-
-    def __init__(self, parent:QWidget, licenseFile:Optional[str]) ->None:
-        KDialog.__init__(self, parent)
-        self.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose)
-        self.setCaption(i18n("License Agreement"))
-        self.setButtons(KDialog.Close)
-        self.buttonBox.setFocus()
-        if licenseFile is None:
-            licenseText = 'no license file found'
-        else:
-            with open('x' + licenseFile, 'r', encoding='utf-8') as _:
-                licenseText = _.read()
-        self.licenseBrowser = QTextBrowser()
-        self.licenseBrowser.setLineWrapMode(QTextEdit.LineWrapMode.NoWrap)
-        self.licenseBrowser.setText(licenseText)
-
-        vLayout = QVBoxLayout()
-        vLayout.addWidget(self.licenseBrowser)
-        vLayout.addWidget(self.buttonBox)
-        self.setLayout(vLayout)
-
-    def sizeHint(self) ->QSize:
-        """try to set up the dialog such that the full width of the
-        document is visible without horizontal scroll-bars being required"""
-        idealWidth = self.licenseBrowser.document().idealWidth()
-        idealWidth += self.style().pixelMetric(QStyle.PixelMetric.PM_LayoutLeftMargin)
-        idealWidth += self.style().pixelMetric(QStyle.PixelMetric.PM_LayoutRightMargin)
-        idealWidth += self.licenseBrowser.verticalScrollBar().width() * 2 + 1
-        # try to allow enough height for a reasonable number of lines to be
-        # shown
-        metrics = QFontMetrics(self.licenseBrowser.font())
-        idealHeight = metrics.height() * 30
-        return KDialog.sizeHint(self).expandedTo(QSize(int(idealWidth), idealHeight))
-
 
 class KConfigDialog(KDialog):
 
@@ -1201,7 +1148,7 @@ class KConfigDialog(KDialog):
         'QLineEdit': 'setText'}
 
     def __init__(self, parent:QWidget, name:str, preferences:'SetupPreferences') ->None:
-        KDialog.__init__(self, parent)
+        super().__init__(parent)
         self.pages: List[QWidget]
         self.setCaption(i18n('Configure'))
         self.name = name
@@ -1217,9 +1164,8 @@ class KConfigDialog(KDialog):
         self.tabSpace = QStackedWidget()
         self.setButtons(KDialog.Help | KDialog.Ok |
                         KDialog.Apply | KDialog.Cancel | KDialog.RestoreDefaults)
-        self.buttonBox.button(
-            KDialog.Apply).clicked.connect(
-                self.applySettings)
+        if button := self.buttonBox.button(KDialog.Apply):
+            button.clicked.connect(self.applySettings)
         cmdLayout = QHBoxLayout()
         cmdLayout.addWidget(self.buttonBox)
         self.contentLayout = QHBoxLayout()
@@ -1241,7 +1187,7 @@ class KConfigDialog(KDialog):
             return cls.dialog
         return None
 
-    def showEvent(self, unusedEvent:QEvent) ->None:
+    def showEvent(self, unusedEvent:Optional[QEvent]) ->None:
         """if the settings dialog shows, remember current values
         and show them in the widgets"""
         self.orgPref = self.preferences.as_dict() # FIXME: unused
@@ -1306,7 +1252,7 @@ class KConfigDialog(KDialog):
         """OK pressed"""
         if self.updateButtons():
             self.updateSettings()
-        KDialog.accept(self)
+        super().accept()
 
     def updateButtons(self) ->bool:
         """Updates the Apply and Default buttons. Returns True if there was a changed setting"""
@@ -1319,7 +1265,8 @@ class KConfigDialog(KDialog):
             if oldValue != newValue:
                 changed = True
                 break
-        self.buttonBox.button(KDialog.Apply).setEnabled(changed)
+        if button := self.buttonBox.button(KDialog.Apply):
+            button.setEnabled(changed)
         return changed
 
     def updateSettings(self) ->None:
@@ -1341,8 +1288,8 @@ class KConfigDialog(KDialog):
         """show wanted page and select its icon"""
         self.tabSpace.setCurrentWidget(page)
         for idx in range(self.tabSpace.count()):
-            self.iconList.item(idx).setSelected(
-                idx == self.tabSpace.currentIndex())
+            if item := self.iconList.item(idx):
+                item.setSelected(idx == self.tabSpace.currentIndex())
 
 
 class KSeparator(QFrame):
@@ -1350,7 +1297,7 @@ class KSeparator(QFrame):
     """used for toolbar editor"""
 
     def __init__(self, parent:QWidget) ->None:
-        QFrame.__init__(self, parent)
+        super().__init__(parent)
         self.setLineWidth(1)
         self.setMidLineWidth(0)
         self.setFrameShape(QFrame.Shape.HLine)
@@ -1366,7 +1313,7 @@ class ToolBarItem(QListWidgetItem):
         self.action = action
         self.parent = parent
         self.emptyIcon:QIcon
-        QListWidgetItem.__init__(self, self.__icon(), self.__text(), parent)
+        super().__init__(self.__icon(), self.__text(), parent)
         # drop between items, not onto items
         self.setFlags(
             cast(Qt.ItemFlag, (self.flags() | Qt.ItemFlag.ItemIsDragEnabled) & ~Qt.ItemFlag.ItemIsDropEnabled))
@@ -1376,11 +1323,12 @@ class ToolBarItem(QListWidgetItem):
         result = self.action.icon()
         if result.isNull():
             if not hasattr(self, 'emptyIcon'):
-                iconSize = self.parent.style().pixelMetric(
-                    QStyle.PixelMetric.PM_SmallIconSize)
-                _ = QPixmap(iconSize, iconSize)
-                _.fill(Qt.GlobalColor.transparent)
-                self.emptyIcon = QIcon(_)
+                if style := self.parent.style():
+                    iconSize = style.pixelMetric(
+                        QStyle.PixelMetric.PM_SmallIconSize)
+                    _ = QPixmap(iconSize, iconSize)
+                    _.fill(Qt.GlobalColor.transparent)
+                    self.emptyIcon = QIcon(_)
             result = self.emptyIcon
         return result
 
@@ -1394,7 +1342,7 @@ class ToolBarList(QListWidget):
     """QListWidget without internal moves"""
 
     def __init__(self, parent:QWidget) ->None:
-        QListWidget.__init__(self, parent)
+        super().__init__(parent)
         self.setDragDropMode(QAbstractItemView.DragDropMode.DragDrop)  # no internal moves
 
 
@@ -1404,7 +1352,7 @@ class KEditToolBar(KDialog):
 
     def __init__(self, parent:Optional[QWidget]=None) ->None:
         # pylint: disable=too-many-statements
-        KDialog.__init__(self, parent)
+        super().__init__(parent)
         self.setCaption(i18n('Configure Toolbars'))
         StateSaver(self)
         self.inactiveLabel = QLabel(i18n("A&vailable actions:"), self)
